@@ -163,7 +163,7 @@ GLOBAL0(signed char be_customer_proof);
 #ifdef WITH_TLS
 GLOBAL0(signed char enforce_tls_auth);
 GLOBAL0(int data_protection_level);
-GLOBAL(const char *tlsciphersuite, NULL);
+GLOBAL(const char *tlsciphersuite, TLS_DEFAULT_CIPHER_SUITE);
 GLOBAL0(signed char ssl_disabled);
 GLOBAL0(signed char ssl_verify_client_cert);
 #endif
